@@ -1,16 +1,17 @@
 # place the root of a tree in the middle of an edge (creating a new node)
-root_in_edge = function(tree, root_edge, new_root_name=NULL, collapse_monofurcations=TRUE){
+root_in_edge = function(tree, root_edge, location=0.5, new_root_name=NULL, collapse_monofurcations=TRUE){
 	Ntips 		= length(tree$tip.label)
 	Nnodes 		= tree$Nnode
 	Nedges 		= nrow(tree$edge)
 	new_root 	= Ntips+Nnodes+1 # append new root at the end of the node list
+	location	= max(0,min(1,location))
 	
 	# split edge (downstream half becomes a new edge)
 	tree$edge = rbind(tree$edge, c(new_root, tree$edge[root_edge,2]))
 	tree$edge[root_edge,2] = new_root
 	if(!is.null(tree$edge.length)){
-		tree$edge.length = c(tree$edge.length, 0.5*tree$edge.length[root_edge])
-		tree$edge.length[root_edge] = 0.5*tree$edge.length[root_edge]
+		tree$edge.length = c(tree$edge.length, (1-location)*tree$edge.length[root_edge])
+		tree$edge.length[root_edge] = location*tree$edge.length[root_edge]
 	}
 	
 	# add new node
